use std::{io, path::PathBuf};

use serde::{Deserialize, Serialize};

use crate::{
    backup::{ManagerConfig, MetaManager},
    server::{Metadata, ServerType},
};

#[derive(Serialize, Deserialize, Debug)]
pub struct Config {
    pub config: PathBuf,
    pub world: PathBuf,
    pub backup: PathBuf,
    pub layers: Vec<ManagerConfig>,
    pub server: ServerType,
    pub server_version: String,
}

impl Default for Config {
    fn default() -> Self {
        Self {
            config: PathBuf::from("."),
            world: PathBuf::from("../worlds"),
            backup: PathBuf::from("../backups"),
            layers: Vec::new(),
            server: ServerType::Unknown,
            server_version: String::from(""),
        }
    }
}

impl Config {
    /// Convenience method to initialize backup manager from the cli arguments
    pub fn meta(&self) -> io::Result<MetaManager<Metadata>> {
        let metadata = Metadata {
            server_type: self.server,
            server_version: self.server_version.clone(),
        };
        let dirs = vec![
            (PathBuf::from("config"), self.config.canonicalize()?),
            (PathBuf::from("worlds"), self.world.canonicalize()?),
        ];
        let mut meta = MetaManager::new(self.backup.canonicalize()?, dirs, metadata);
        meta.add_all(&self.layers)?;

        Ok(meta)
    }
}
